mod Maux logic: 'cl'

incl Mtex

rem 
  \para \bf  Numeric programs.  \end 

rem 
  \para \it  Maximum.  \end 

fun/2 Max 'Tex_f2_n_max'
  Max(x,y) = x <- x >= y
  Max(x,y) = y <- x < y

thm Max_comm
  Max(x,y) = Max(y,x)
proof 
 case Trich; x,y @ x1,y1; x1,y1; x1,y1
  proved
  proved
  proved..

rem 
  \para \it  Exponentiation.  \end 

fun/2 Exp 'Tex_f2_n_exp'
  Exp(x,0) = 1
  Exp(x,y+1) = x*Exp(x,y)

thm Exp_is_zero
  Exp(x,y) = 0 <-> x = 0 & y != 0
proof 
 ind N; y @ 0; y
  proved
  weak Exp(x,y) = 0 <-> x = 0 & y != 0
   cut x = 0
    proved
    proved....

thm Exp_is_one
  Exp(x,y) = 1 <-> x = 1 \/ y = 0
proof 
 ind N; y @ 0; y
  proved
  split* x = 1 & (x = 1 \/ y = 0) <-> x = 1.2,0,0
   proved
   proved...

thm Exp_plus
  Exp(x,y+z) = Exp(x,y)*Exp(x,z)
proof 
 ind N; y @ 0; y
  proved
  proved..

thm Exp_minus
  x != 0 & y >= z -> Exp(x,y-z) = Exp(x,y)/Exp(x,z)
proof 
 ind N; y; z @ 0; y
  proved
  case N; z @ 0; z1
   proved
   inst \a z(y >= z -> Exp(x,y-z) = Exp(x,y)/Exp(x,z)); z1 proved....

thm Exp_cancel_eq
  x > 1 -> Exp(x,y) = Exp(x,z) <-> y = z
proof 
 ind N; y; z @ 0; y
  use Exp_is_one.0; x; z proved.
  case N; z @ 0; z1
   proved
   inst \a z(Exp(x,y) = Exp(x,z) <-> y = z).0; z1 proved....

thm Exp_cancel_le
  x > 1 -> Exp(x,y) <= Exp(x,z) <-> y <= z
proof 
 ind N; y; z @ 0; y
  use Exp_is_zero.0; x; z proved.
  case N; z @ 0; z1
   use Exp_is_zero.0; x; y proved.
   inst \a z(Exp(x,y) <= Exp(x,z) <-> y <= z).0; z1 proved....

thm Exp_cancel_lt
  x > 1 -> Exp(x,y) < Exp(x,z) <-> y < z
proof 
 ind N; y; z @ 0; y
  cut z = 0
   proved
   use Exp_is_zero.0; x; z-1 proved..
  case N; z @ 0; z1
   use Exp_is_zero.0; x; y proved.
   inst \a z(Exp(x,y) < Exp(x,z) <-> y < z).0; z1 proved....

rem 
  \para \bf  Lists.  \end 

rem 
  \para \it  List length.  \end 

fun L 
  L(0) = 0
  L(v,w) = L(w)+1

thm L_zero
  L(x) = 0 <-> x = 0
proof 
 case Ln; x @ 0; v,w
  proved
  proved..

thm L_conc
  L(x++y) = L(x)+L(y)
proof 
 ind Ln; x @ 0; v,w
  proved
  proved..

rem 
  \para \it  List indexing.  \end 

fun/2 Sub 'Tex_f2_ln_sub'
  Sub((v,w),0) = v
  Sub((v,w),i+1) = Sub(w,i)

rem 
  \para \it  List reversal.  \end 

fun Rev 
  Rev(0) = 0
  Rev(v,w) = Rev(w)++(v,0)

thm Rev_conc
  Rev(x++y) = Rev(y)++Rev(x)
proof 
 ind Ln; x @ 0; v,w
  proved
  proved..

rem 
  \para \it  Half-open interval.  \end 

fun/2 Interval 'Tex_f2_ln_interval_co'
  Interval(m,n) = 0 <- m >= n
  Interval(m,n) = m,Interval(m+1,n) <- m < n

thm Interval_l
  L Interval(m,n) = n-m
proof 
 indm n-m @ n1,m1
  case Dich; n,m @ x,y; x,y
   proved
   inst \a n1\a m1(m+(n1-m1) < n -> L Interval(m1,n1) = n1-m1); n; m+1 proved....

thm Interval_sub
  m+i < n -> Sub(Interval(m,n),i) = m+i
proof 
 indm n-m; i @ n1,m1
  case Dich; n,m+1 @ x,y; x,y
   proved
   case N; i @ 0; i1
    proved
    inst \a n1\a m1(m+(n1-m1) < n -> 
                    \a i(i+m1 < n1 -> Sub((m1,Interval(m1+1,n1)),i) = i+m1)):0,2,
                                                                             1,0; n; m+1; i1 proved.....

thm Interval_conc
  k <= m & m <= n -> Interval(k,m)++Interval(m,n) = Interval(k,n)
proof 
 indm m-k @ m1,k1
  case Dich; m,k @ x,y; x,y
   proved
   inst \a m1\a k1(k+(m1-k1) < m -> k1 <= m1 & m1 <= n -> 
                   Interval(k1,m1)++Interval(m1,n) = Interval(k1,n)); m; k+1 proved....

